@@ -23,17 +23,16 @@
 
 #include <CGAL/license/Arrangement_on_surface_2.h>
 
+#define CGAL_DEPRECATED_HEADER "<CGAL/Arr_overlay.h>"
+#define CGAL_REPLACEMENT_HEADER "<CGAL/Arr_overlay_2.h>"
+#include <CGAL/internal/deprecation_warning.h>
+
+#include <CGAL/Arr_overlay_2.h>
 
 /*! \file
  * Helping file to include Arr_overlay_2 for backward compatibility.
  */
 
-#if (defined __GNUC__)
-  #warning Arr_overlay.h is DEPRECATED, please include Arr_overlay_2.h instead
-#elif (defined _MSC_VER)
-  #pragma message("Arr_overlay.h is DEPRECATED, please include Arr_overlay_2.h instead")
-#endif
-
 #include <CGAL/Arr_overlay_2.h>
 
 #endif
